#!/usr/bin/env python
#----------------------------------------------------------------------------
# Project Name: wxDick
# File Name: config.py
# Description: Configuration class.
#
# Created: 2004. 10. 04
# RCS-ID: $Id: config.py,v 1.22 2004/10/22 14:12:34 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

import wx
import wx.lib.mixins.listctrl as listmix
import wx.lib.intctrl
import wx.lib.newevent
import dictclient
import os
import sys
import copy
import string
import gettext
import threading
import ConfigParser

gettext.install("wxDick")

#----------------------------------------------------------------------------

(UpdateDatabaseEvent, EVT_UPDATE_DATABASE) = wx.lib.newevent.NewEvent()
(UpdateStrategyEvent, EVT_UPDATE_STRATEGY) = wx.lib.newevent.NewEvent()

#----------------------------------------------------------------------------

class GetDatabase(threading.Thread):
    def __init__(self, parent, hostname = "localhost", port = "2628"):
        threading.Thread.__init__(self)
        self.dialog = parent
        self.hostname = hostname
        self.port = int(port)
        self.running = False
        self.connection = None
        self.database = None
    
    def IsRunning(self):
        return self.running

    def run(self):
        self.running = True

        try:
            self.connection = dictclient.Connection(self.hostname, self.port)
            self.database = self.connection.getdbdescs()
        except:
            pass
        
        if self.running:
            event = UpdateDatabaseEvent(database = self.database)
            wx.PostEvent(self.dialog, event)

        self.running = False
    	
    def stop(self):
        self.running = False    

#----------------------------------------------------------------------------

class GetStrategy(threading.Thread):
    def __init__(self, parent, hostname = "localhost", port = "2628"):
        threading.Thread.__init__(self)
        self.dialog = parent
        self.hostname = hostname
        self.port = int(port)
        self.running = False
        self.connection = None
        self.strategy = None
    
    def IsRunning(self):
        return self.running

    def run(self):
        self.running = True

        try:
            self.connection = dictclient.Connection(self.hostname, self.port)
            self.strategy = self.connection.getstratdescs()
        except:
            pass
        
        if self.running:
            event = UpdateStrategyEvent(strategy = self.strategy)
            wx.PostEvent(self.dialog, event)

        self.running = False
    	
    def stop(self):
        self.running = False

#----------------------------------------------------------------------------

class ServerInfo(wx.Dialog):
    def __init__(
        self, parent, id, title, key = None, size = wx.DefaultSize, pos = wx.DefaultPosition, 
        style = wx.DEFAULT_DIALOG_STYLE
        ):
        pre = wx.PreDialog()
        pre.Create(parent, id, title, pos, size, style)

        self.PostCreate(pre)
        self.databaseThread = None
        self.strategyThread = None

        if not key == None:
            self.server = copy.deepcopy(parent.servers[key])
            self.labelCtrl = wx.TextCtrl(self, -1, self.server[0], size = (150, -1))                
            self.addressCtrl = wx.TextCtrl(self, -1, self.server[1], size = (150, -1))                
            self.portCtrl = wx.lib.intctrl.IntCtrl(self, -1, int(self.server[2]), size = (150, -1))
            self.databaseCtrl = wx.ComboBox(self, -1, value = self.server[3], choices = [self.server[3]],
                                style = wx.CB_DROPDOWN | wx.CB_READONLY, size = (150, -1))
            self.strategyCtrl = wx.ComboBox(self, -1, value = self.server[4], choices = [self.server[4]],
                                style = wx.CB_DROPDOWN | wx.CB_READONLY, size = (150, -1))
        else:
            self.server = copy.deepcopy(parent.defaultServer)
            self.labelCtrl = wx.TextCtrl(self, -1, self.server[0], size = (150, -1))                
            self.addressCtrl = wx.TextCtrl(self, -1, self.server[1], size = (150, -1))                
            self.portCtrl = wx.lib.intctrl.IntCtrl(self, -1, int(self.server[2]), size = (150, -1))
            self.databaseCtrl = wx.ComboBox(self, -1, value = self.server[3], choices = [self.server[3]],
                                style = wx.CB_DROPDOWN | wx.CB_READONLY , size = (150, -1))
            self.strategyCtrl = wx.ComboBox(self, -1, value = self.server[4], choices = [self.server[4]],
                                style = wx.CB_DROPDOWN | wx.CB_READONLY , size = (150, -1))
        
        self.labelText = wx.StaticText(self, -1, _("Label"), size = (60, -1))
        self.addressText = wx.StaticText(self, -1, _("Address"), size = (60, -1))                
        self.portText = wx.StaticText(self, -1, _("Port"), size = (60, -1))
        self.databaseText = wx.StaticText(self, -1, _("Database"), size = (60, -1))
        self.databaseButton = wx.Button(self, -1, _("Update"))
        self.strategyText = wx.StaticText(self, -1, _("Strategy"), size = (60, -1))
        self.strategyButton = wx.Button(self, -1, _("Update"))

        self.line = wx.StaticLine(self, -1, style = wx.LI_HORIZONTAL)
        self.okButton = wx.Button(self, wx.ID_OK, _("OK"))
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, _("Cancel"))
        self.okButton.SetDefault()

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        
        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.labelText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.labelCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.addressText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.addressCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.portText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.portCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.databaseText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.databaseCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.hSizer.Add(self.databaseButton, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.strategyText, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.strategyCtrl, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.hSizer.Add(self.strategyButton, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_LEFT | wx.ALL, 1)

        self.sizer.Add(self.line, 0, wx.EXPAND | wx.ALIGN_CENTER_VERTICAL | wx.LEFT | wx.RIGHT, 5)
        
        self.hSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.hSizer.Add(self.okButton, 0, wx.ALIGN_CENTRE | wx.ALL, 4)
        self.hSizer.Add(self.cancelButton, 0, wx.ALIGN_CENTRE | wx.BOTTOM | wx.TOP | wx.RIGHT, 4)
        self.sizer.Add(self.hSizer, 0, wx.ALIGN_RIGHT | wx.ALL, 1)
        
        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        self.sizer.Fit(self)

        self.Bind(wx.EVT_TEXT, self.OnEditLabel, self.labelCtrl)
        self.Bind(wx.EVT_TEXT, self.OnEditAddress, self.addressCtrl)
        self.Bind(wx.lib.intctrl.EVT_INT, self.OnEditPort, self.portCtrl)
        self.Bind(wx.EVT_COMBOBOX, self.OnEditDatabase, self.databaseCtrl)                
        self.Bind(wx.EVT_COMBOBOX, self.OnEditStrategy, self.strategyCtrl)
        self.Bind(wx.EVT_BUTTON, self.OnDatabaseButton, self.databaseButton)
        self.Bind(wx.EVT_BUTTON, self.OnStrategyButton, self.strategyButton)
        self.Bind(EVT_UPDATE_DATABASE, self.OnUpdateDatabase)
        self.Bind(EVT_UPDATE_STRATEGY, self.OnUpdateStrategy)

    def GetServerInfo(self):
        return self.server

    def OnEditLabel(self, event):
        self.server[0] = event.GetString()
		
    def OnEditAddress(self, event):
        self.server[1] = event.GetString()
		
    def OnEditPort(self, event):
        self.server[2] = str(event.GetValue())

    def OnEditDatabase(self, event):
        try:
            data = self.databaseCtrl.GetClientData(self.databaseCtrl.GetSelection())
        except:
            return

        self.databaseCtrl.SetToolTip(wx.ToolTip(data))
        self.server[3] = event.GetString()        

    def OnEditStrategy(self, event):
        try:
            data = self.strategyCtrl.GetClientData(self.strategyCtrl.GetSelection())
        except:
            return

        self.strategyCtrl.SetToolTip(wx.ToolTip(data))
        self.server[4] = event.GetString()

    def OnDatabaseButton(self, event):
        if not self.server[1]:
            dialog = wx.MessageDialog(self, _("Please Enter Address"), "Warning",
				       wx.OK | wx.ICON_INFORMATION)
	    dialog.ShowModal()
	    dialog.Destroy()
        else:
            if self.databaseThread and self.databaseThread.IsRunning():
                self.databaseThread.stop()
            
            self.databaseThread = GetDatabase(self, self.server[1], self.server[2])
            self.databaseThread.start()

    def OnStrategyButton(self, event):
        if not self.server[1]:
            dialog = wx.MessageDialog(self, _("Please Enter Address"), _("Warning"),
				       wx.OK | wx.ICON_INFORMATION)
	    dialog.ShowModal()
	    dialog.Destroy()
        else:
            if self.strategyThread and self.strategyThread.IsRunning():
                self.strategyThread.stop()
            
            self.strategyThread = GetStrategy(self, self.server[1], self.server[2])
            self.strategyThread.start()
                    
    def OnUpdateDatabase(self, event):
        if event.database:
            self.databaseCtrl.Clear()
            self.databaseCtrl.Append("*", _("Search all databases"))
            self.databaseCtrl.Append("!", _("The first database that has a match"))

            for key, value in event.database.items() :
                self.databaseCtrl.Append(key, value)

            self.databaseCtrl.SetSelection(0)
            self.databaseCtrl.SetToolTip(wx.ToolTip(self.databaseCtrl.GetClientData(0)))
            self.server[3] = self.databaseCtrl.GetValue()
            
    def OnUpdateStrategy(self, event):
        if event.strategy:
            self.strategyCtrl.Clear()

            for key, value in event.strategy.items() :
                self.strategyCtrl.Append(key, value)

            self.strategyCtrl.SetSelection(0)
            self.strategyCtrl.SetToolTip(wx.ToolTip(self.strategyCtrl.GetClientData(0)))
            self.server[4] = self.strategyCtrl.GetValue()
    	
#----------------------------------------------------------------------------

class MyServerList(wx.ListCtrl, listmix.ListCtrlAutoWidthMixin):
    def __init__(self, parent, id, pos = wx.DefaultPosition,
                         size = wx.DefaultSize, style = wx.LC_REPORT
                                                         | wx.BORDER_NONE
                                                         | wx.LC_SINGLE_SEL 
                                                         | wx.LC_SORT_ASCENDING):
        wx.ListCtrl.__init__(self, parent, id, pos, size, style)
        listmix.ListCtrlAutoWidthMixin.__init__(self)

        self.frame = parent.frame
        self.currentItem = None
        self.selected = False
        self.servers = copy.deepcopy(self.frame.config.servers)
        self.defaultServer = copy.deepcopy(self.frame.config.defaultServer)
        self.PopulateList(self.servers)
        self.GetImages()
 
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected)
        self.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.OnRightClick)
        self.Bind(wx.EVT_RIGHT_UP, self.OnRightClick)
        self.Bind(wx.EVT_RIGHT_DOWN, self.OnRightDown)

    def GetImages(self, theme = "default"):
        if wx.Platform == "__WXMSW__":
            self.edit = wx.Image(os.path.join("pixmaps", theme + "_edit.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.add = wx.Image(os.path.join("pixmaps", theme + "_add.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.remove = wx.Image(os.path.join("pixmaps", theme + "_remove.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()  
        else:
            prefix = os.path.join(sys.prefix, "share", "wxdick")
            self.edit = wx.Image(os.path.join(prefix, "pixmaps", theme + "_edit.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.add = wx.Image(os.path.join(prefix, "pixmaps", theme + "_add.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.remove = wx.Image(os.path.join(prefix, "pixmaps", theme + "_remove.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()

    def OnRightDown(self, event):
        self.x = event.GetX()
        self.y = event.GetY()
        item, flags = self.HitTest((self.x, self.y))

        if flags & wx.LIST_HITTEST_ONITEM:
            self.Select(item)
            self.selected = True
        else:
            self.selected = False

        event.Skip()

    def OnRightClick(self, event):
        if not hasattr(self, "popupEditServer"):
            self.popupEditServer = wx.NewId()
            self.popupRemoveServer = wx.NewId()
            self.popupAddServer = wx.NewId()

            self.Bind(wx.EVT_MENU, self.OnPopupEditServer, id = self.popupEditServer)
            self.Bind(wx.EVT_MENU, self.OnPopupRemoveServer, id = self.popupRemoveServer)            
            self.Bind(wx.EVT_MENU, self.OnPopupAddServer, id = self.popupAddServer)

        menu = wx.Menu()

        if self.selected:
            item = wx.MenuItem(menu, self.popupEditServer, _("&Edit Server"))
            item.SetBitmap(self.edit)
            menu.AppendItem(item)
            item = wx.MenuItem(menu, self.popupRemoveServer, _("&Remove Server"))
            item.SetBitmap(self.remove)
            menu.AppendItem(item)
        else:
            item = wx.MenuItem(menu, self.popupAddServer, _("&Add Server"))
            item.SetBitmap(self.add)
            menu.AppendItem(item)

        self.PopupMenu(menu, (self.x, self.y))
        menu.Destroy()

    def OnPopupEditServer(self, event):
        key = self.GetItemData(self.currentItem)
        dialog = ServerInfo(self, -1, _("Edit Server"), key)

        if dialog.ShowModal() == wx.ID_OK:
            serverInfo = dialog.GetServerInfo()
            self.servers[key] = serverInfo
            self.PopulateList(self.servers, True)

        dialog.Destroy()

    def OnPopupRemoveServer(self, event):
        del self.servers[self.GetItemData(self.currentItem)]
        self.PopulateList(self.servers, True)

    def OnPopupAddServer(self, event):
        dialog = ServerInfo(self, -1, _("Add Server"))

        if dialog.ShowModal() == wx.ID_OK:
            serverInfo = dialog.GetServerInfo()
            self.servers.append(serverInfo)
            self.PopulateList(self.servers, True)

        dialog.Destroy()

    def OnItemSelected(self, event):
        self.currentItem = event.m_itemIndex
        event.Skip()

    def PopulateList(self, servers, renew = False):
        if not renew:
            self.InsertColumn(0, _("Label"))
            self.InsertColumn(1, _("Address"))
            self.InsertColumn(2, _("Port"))
            self.InsertColumn(3, _("Database"))
            self.InsertColumn(4, _("Strategy"))

            self.SetColumnWidth(0, wx.LIST_AUTOSIZE_USEHEADER)
            self.SetColumnWidth(1, wx.LIST_AUTOSIZE_USEHEADER)
            self.SetColumnWidth(2, wx.LIST_AUTOSIZE_USEHEADER)
            self.SetColumnWidth(3, wx.LIST_AUTOSIZE_USEHEADER)
            self.SetColumnWidth(4, wx.LIST_AUTOSIZE_USEHEADER)

        self.DeleteAllItems()

        for key in range(len(servers)):
            index = self.InsertStringItem(sys.maxint, servers[key][0])
            self.SetStringItem(index, 1, servers[key][1])
            self.SetStringItem(index, 2, servers[key][2])
            self.SetStringItem(index, 3, servers[key][3])
            self.SetStringItem(index, 4, servers[key][4])
            self.SetItemData(index, key)

#----------------------------------------------------------------------------

class MyPreferences(wx.Dialog):
    def __init__(
            self, parent, id, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
            style = wx.DEFAULT_DIALOG_STYLE
            ):
        pre = wx.PreDialog()
        pre.Create(parent, id, title, pos, size, style)

        self.PostCreate(pre)

        self.frame = parent
        self.stayOnTop = self.frame.config.stayOnTop
        self.stayOnTopCtrl = wx.CheckBox(self, -1, _("New opened wxDick stay on top"), style = wx.NO_BORDER)
        self.stayOnTopCtrl.SetValue(self.stayOnTop)
        self.serverList = MyServerList(self, -1, size = (300, 100))

        self.cancelButton = wx.Button(self, wx.ID_CANCEL, _("Cancel"))
        self.okButton = wx.Button(self, wx.ID_OK, _("OK"))
        self.okButton.SetDefault()

        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.staticBox = wx.StaticBox(self, -1, _("Stay on Top"))
        self.staticSizer = wx.StaticBoxSizer(self.staticBox, wx.VERTICAL)
        self.staticSizer.Add(self.stayOnTopCtrl, 0, wx.ALIGN_LEFT | wx.ALL, 5)
        self.sizer.Add(self.staticSizer, 0, wx.EXPAND | wx.ALL, 5)        
        self.staticBox = wx.StaticBox(self, -1, _("Server"))
        self.staticSizer = wx.StaticBoxSizer(self.staticBox, wx.VERTICAL)
        self.staticSizer.Add(self.serverList, 1, wx.EXPAND | wx.ALIGN_CENTRE | wx.ALL, 5)
        self.sizer.Add(self.staticSizer, 0, wx.EXPAND | wx.ALL, 5)       
        self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.buttonSizer.Add(self.okButton, 0, wx.ALIGN_CENTRE | wx.ALL, 5)
        self.buttonSizer.Add(self.cancelButton, 0, wx.ALIGN_CENTRE | wx.TOP | wx.BOTTOM | wx.RIGHT, 5)

        self.sizer.Add(self.buttonSizer, 0, wx.ALIGN_RIGHT | wx.LEFT | wx.RIGHT | wx.BOTTOM, 0)
        self.Bind(wx.EVT_BUTTON, self.OnButton, self.okButton, id = wx.ID_OK)

        self.SetSizer(self.sizer)
        self.SetAutoLayout(True)
        self.sizer.Fit(self)
    
        self.Bind(wx.EVT_CHECKBOX, self.OnStayOnTop, self.stayOnTopCtrl)

    def OnStayOnTop(self, event):
        self.stayOnTop = event.IsChecked()
    	
    def OnButton(self, event):
        self.frame.config.servers = self.serverList.servers
        self.frame.config.stayOnTop = self.stayOnTop
        self.frame.config.Save()
        self.EndModal(wx.ID_OK)
    	
#----------------------------------------------------------------------------

class Config:
    def __init__(self, configFile):
        self.configFile = os.path.realpath(configFile)
        self.defaultServer = ["DICT.ORG", "dict.org", "2628", "*", "exact"]
        self.Load()

    def Save(self):
        config = ConfigParser.ConfigParser()
        config.add_section("general")
        config.add_section("server")

        config.set("general", "stay-on-top", self.stayOnTop)
        
        for key in range(len(self.servers)):            
            config.set("server", "server-info%d" % key, string.join(self.servers[key], " :: "))
    	
        configFile = open(self.configFile, "w")
        config.write(configFile)
        configFile.close()

    def Load(self):
        config = ConfigParser.ConfigParser()
        config.add_section("general")
        config.add_section("server")
        
        config_defaults = config.defaults()
        config_defaults["stay-on-top"] = "False"
        
        config.read(self.configFile)

        self.stayOnTop = config.getboolean("general", "stay-on-top")
        
        servers = []
        self.servers = []

        for option in config.options("server"):
	    if option[:11] == "server-info":
	        servers.append((int(option[11:]), config.get("server", option)))

        if servers:
            servers.sort()

            for key in range(len(servers)):            
                defaultServer = copy.deepcopy(self.defaultServer)
                server = map(string.strip, string.split(servers[key][1], "::"))

                for i in range(len(server)):
                    defaultServer[i] = server[i]

                self.servers.append(defaultServer)           
