#!/usr/bin/env python
#----------------------------------------------------------------------------
# Project Name: wxDick
# File Name: setup.py
# Description: Installation script.
#
# Created: 2004. 10. 21
# RCS-ID: $Id: setup.py,v 1.5 2004/10/23 10:13:12 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

import sys

try:
    if sys.version_info < (2, 3):
        print "wxDick requires Python 2.3 or higher"
        sys.exit(1)
except AttributeError:
    print "wxDick requires Python 2.3 or higher"
    sys.exit(1)

import os
import glob
from distutils.core import setup
from distutils.command.install_lib import install_lib
from distutils.command.install_data import install_data
from wxdick import __version__


#----------------------------------------------------------------------------

installed_lib_dir = None

#----------------------------------------------------------------------------

class my_install_lib(install_lib):
    def finalize_options(self):
        global installed_lib_dir
        install_lib.finalize_options(self)
        installed_lib_dir = self.install_dir

#----------------------------------------------------------------------------

class my_install_data(install_data):
    def finalize_options(self):
        global installed_lib_dir
        install_data.finalize_options(self)
        
        if not os.path.isdir(os.path.join(self.install_dir, "bin")):
            print "creating %s" % os.path.join(self.install_dir, "bin")
            os.mkdir(os.path.join(self.install_dir, "bin"))
            
        if not os.path.isfile(os.path.join(self.install_dir, "bin", "wxdick")):
            print "changing permission %s to 0755" % (os.path.join(installed_lib_dir, "wxdick", "wxdick.py"))
            os.chmod(os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), 0755)
            print "symbolic linking %s -> %s" % (os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), os.path.join(self.install_dir, "bin", "wxdick"))
            os.symlink(os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), os.path.join(self.install_dir, "bin", "wxdick"))
        else:
            if self.force:
                print "changing permission %s to 0755" % (os.path.join(installed_lib_dir, "wxdick", "wxdick.py"))
                os.chmod(os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), 0755)
                print "symbolic linking %s -> %s" % (os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), os.path.join(self.install_dir, "bin", "wxdick"))
                os.remove(os.path.join(self.install_dir, "bin", "wxdick"))
                os.symlink(os.path.join(installed_lib_dir, "wxdick", "wxdick.py"), os.path.join(self.install_dir, "bin", "wxdick"))

                
#----------------------------------------------------------------------------

if __name__ == "__main__":
    if sys.platform != "win32":
        setup(
        name = "wxdick",
        version = __version__,
        license = "GNU Generic Public License",
        url = "http://wxdick.kldp.net",
        author = "Myung-Gon Park",
        author_email = "myunggoni@users.kldp.net",
        platforms = ["OS Independent"],
        keywords = ["Dictionary"],
        description = "DICT Protocol Based Dictionary Server/Client for All Platforms Written in wxPython",
        long_description = "wxDick is DICT protocol based dictionary server/client for all platforms written in wxPython.\n",
        download_url = "http://kldp.net/project/showfiles.php?group_id=520",
        packages = ["wxdick"],
        package_dir = {"wxdick": ""},
        data_files = [
        ("share/wxdick/conf", ["conf/wxdick.conf.sample"]),
        ("share/wxdick/pixmaps", glob.glob("pixmaps/*.*")),],
        cmdclass = {"install_lib" : my_install_lib,
                "install_data" : my_install_data}
        )
    else:
        try:
            sys.argv[1]
        except IndexError:
            print "usage: %s py2exe [option]" % sys.argv[0]
            sys.exit(1)

        if sys.argv[1] == "py2exe":        
            import py2exe
            setup(
            name = "wxdick",
            version = __version__,
            license = "GNU Generic Public License",
            url = "http://wxdick.kldp.net",
            author = "Myung-Gon Park",
            author_email = "myunggoni@users.kldp.net",
            platforms = ["OS Independent"],
            keywords = ["Dictionary"],
            description = "DICT Protocol Based Dictionary Server/Client for All Platforms Written in wxPython",
            long_description = "wxDick is DICT protocol based dictionary server/client for all platforms written in wxPython.\n",
            download_url = "http://kldp.net/project/showfiles.php?group_id=520",
            packages = ["wxdick"],
            package_dir = {"wxdick": ""},
            windows = [{"script": "wxdick.py",
                    "icon_resources": [(1, "pixmaps/default_wxdick.ico")]
                    }],
            data_files = [
            ("conf", ["conf/wxdick.conf.sample"]),
            ("pixmaps", glob.glob("pixmaps/*.*"))],
            options = {"py2exe": {"packages": ["cjkcodecs"]}}
            )
        else:
            print "wxDick doesn't support installation on Windows."
