#!/usr/bin/env python
#----------------------------------------------------------------------------
# Project Name: wxDick
# File Name: wxdick.py
# Description: Main interface.
#
# Created: 2004. 10. 03
# RCS-ID: $Id: wxdick.py,v 1.50 2004/10/23 09:37:19 myunggoni Exp $
# Copyright: (c) 2004 by myunggoni
# License: GNU General Public License
# Author: Myung-Gon Park <myunggoni@users.kldp.net>
#----------------------------------------------------------------------------

import wx
import wx.lib.newevent
import wx.html as html
import wx.lib.scrolledpanel as scrolled
import dictclient
import sys
import os
import re
import string
import time
import gettext
import locale
import threading
from config import *

if wx.Platform == "__WXMSW__":
    from cjkcodecs import cp949, aliases

    if hasattr(sys, "setdefaultencoding"):
        sys.setdefaultencoding("cp949")

gettext.install("wxDick")
#lang_ko = gettext.translation("wxDick", "./locale", languages=['ko'])
#locale.setlocale(locale.LC_ALL, "") 

#if locale.getlocale()[0] == "Korean_Korea":
#    lang_ko.install()

#----------------------------------------------------------------------------

__package__ = "wxDick"
__version__ = "0.1"
__author__ = _("Myung-Gon Park")
__email__ = "myunggoni@users.kldp.net"

ID_SOUND = wx.NewId()
ID_BACKWARD = wx.NewId()
ID_FORWARD = wx.NewId()
ID_REFRESH = wx.NewId()
ID_STOP = wx.NewId()
ID_PREFERENCES = wx.NewId()
ID_ABOUT = wx.NewId()
ID_EXIT = wx.NewId()

ID_STAYONTOP = wx.NewId()
ID_SHOW = wx.NewId()
ID_HIDE = wx.NewId()

(UpdateResultEvent, EVT_UPDATE_RESULT) = wx.lib.newevent.NewEvent()
(UpdateConnectionEvent, EVT_UPDATE_CONNECTION) = wx.lib.newevent.NewEvent()

#----------------------------------------------------------------------------

class MakeConnection(threading.Thread):
    def __init__(self, parent, hostname = "localhost", port = "2628", onButton = False):
        threading.Thread.__init__(self)
        self.panel = parent
        self.hostname = hostname
        self.port = int(port)
        self.running = False
        self.connection = None
        self.onButton = onButton
    
    def IsRunning(self):
        return self.running

    def run(self):
        self.running = True

        try:
            self.connection = dictclient.Connection(self.hostname, self.port)
        except:
            pass
        
        if self.running:
            event = UpdateConnectionEvent(connection = self.connection, onButton = self.onButton)
            wx.PostEvent(self.panel, event)

        self.running = False
    	
    def stop(self):
        self.running = False    	

#----------------------------------------------------------------------------

class GetDefinition(threading.Thread):
    def __init__(self, parent, connection, word, database = "*", strategy = "exact"):
        threading.Thread.__init__(self)
        self.panel = parent
        self.connection  = connection
        self.database = database
        self.strategy = strategy
        self.word = word
        self.results = []
        self.running = False

    def IsRunning(self):
        return self.running
        
    def run(self):
        self.running = True

        try:
            definitions = self.connection.match(self.database, self.strategy, self.word)
       
            if definitions:
                for definition in definitions:
                    definitionString = definition.getdefstr()
                    definitionString = re.sub(r"{(.*?)}", r"<a href='\1'>\1</a>", definitionString)
                
                    self.results.append("<pre>" + definitionString + "</pre>")
        except:
            self.word = None            

        if self.running:
            event = UpdateResultEvent(word = self.word, results = self.results)
            wx.PostEvent(self.panel, event)

        self.running = False
    
    def stop(self):
        self.running = False	
    	
#----------------------------------------------------------------------------

class MyHtmlWindow(html.HtmlWindow):
    def __init__(self, parent, id):
        html.HtmlWindow.__init__(self, parent, id, style=wx.NO_FULL_REPAINT_ON_RESIZE | wx.SUNKEN_BORDER)
        self.panel = parent
	self.notebook = parent.notebook
	self.frame = parent.frame

        self.SetRelatedFrame(self.frame, __package__)
        self.SetRelatedStatusBar(0)
        self.SetBackgroundColour(wx.WHITE)

        if "gtk2" in wx.PlatformInfo:
            self.NormalizeFontSizes()

    def OnLinkClicked(self, linkinfo):
        self.panel.currentSentence = dictclient.dequote(linkinfo.GetHref())
        self.panel.OnButton(None)

    def OnSetTitle(self, title):
        self.base_OnSetTitle(title)

    def OnCellMouseHover(self, cell, x, y):
        self.base_OnCellMouseHover(cell, x, y)

    def OnCellClicked(self, cell, x, y, evt):
        self.base_OnCellClicked(cell, x, y, evt)

#----------------------------------------------------------------------------

class MyPanel(wx.Panel):
    def __init__(self, parent, id, hostname = "localhost", port = "2628", database = "*", strategy = "exact"):
        wx.Panel.__init__(
            self, parent, id,
            style = wx.TAB_TRAVERSAL | wx.CLIP_CHILDREN | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.notebook = parent
	self.frame = parent.frame
        self.maxHistory = 5
        self.maxWords = 10
        self.previousWords = {}
        self.words = []
        self.previousSentence = []
        self.currentSentence = ""
        self.lastSentence = 0
        self.hostname = hostname
        self.port = port
        self.database = database
        self.strategy = strategy
        self.connection = None
        self.searchThread = None
        self.connectionThread = None
                
        self.MakeConnection()

        self.text = wx.TextCtrl(self, -1)
        self.text.SetInsertionPoint(0)
        self.button = wx.Button(self, -1, _("Search"), size = (80, -1))
	
        line = wx.StaticLine(self, -1, size = (20, -1), style = wx.LI_HORIZONTAL)
        self.resultWindow = MyHtmlWindow(self, -1)
        self.list = wx.ListBox(self, -1, size = (100, -1), style = wx.LB_SINGLE)

        sizer = wx.BoxSizer(wx.VERTICAL)
        searchSizer = wx.BoxSizer(wx.HORIZONTAL)
        resultSizer = wx.BoxSizer(wx.HORIZONTAL)
        
        searchSizer.Add(self.text, 1, wx.EXPAND | wx.ALIGN_RIGHT | wx.LEFT | wx.TOP | wx.BOTTOM, 5)
        searchSizer.Add(self.button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

        resultSizer.Add(self.resultWindow, 1, wx.EXPAND | wx.ALIGN_RIGHT | wx.LEFT | wx.TOP | wx.BOTTOM, 5)
        resultSizer.Add(self.list, 0,  wx.EXPAND | wx.ALIGN_RIGHT | wx.ALL, 5)

        sizer.Add(searchSizer, 0, wx.EXPAND | wx.ALIGN_CENTRE | wx.ALL, 0)
        sizer.Add(line, 0, wx.EXPAND | wx.ALIGN_CENTRE, 0)
        sizer.Add(resultSizer, 1, wx.EXPAND | wx.ALIGN_CENTRE | wx.ALL, 0)

        self.SetSizer(sizer)

        self.Bind(wx.EVT_BUTTON, self.OnButton, self.button)
        self.Bind(wx.EVT_LISTBOX, self.OnListBox, self.list)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.OnListBox, self.list)
        self.Bind(wx.EVT_TEXT, self.OnText, self.text)
        self.Bind(EVT_UPDATE_RESULT, self.OnUpdateResult)
        self.Bind(EVT_UPDATE_CONNECTION, self.OnUpdateConnection)

        self.OnToolSound(None)
        self.OnToolBackward(None)
        self.OnToolForward(None)
        
    def __del__(self):
        if not self.searchThread and not self.connectionThread:
            return
        
        if self.searchThread:
            self.searchThread.stop()

        if self.connectionThread:
            self.connectionThread.stop()            
    	
    def OnButton(self, event):
        if not self.connection:
            self.MakeConnection(True)
            return
        
        if self.words:
            for i in range(len(self.words)):
                self.list.Delete(0)
            self.words = []
 
        for word in string.split(self.currentSentence):
            word = dictclient.dequote(word)

            if word:
                self.words.append(word)

        if self.words:
            if self.lastSentence == len(self.previousSentence):
                self.previousSentence.append(self.words)

                if len(self.previousSentence) > self.maxHistory:
                    trim = len(self.previousSentence) - self.maxHistory
                    self.previousSentence = self.previousSentence[trim:]
                else:
                    self.lastSentence += 1

                    if self.lastSentence > 1:
                        self.frame.toolbar.EnableTool(ID_BACKWARD, 1) 

            else:
                self.previousSentence = self.previousSentence[:self.lastSentence]
                self.lastSentence = len(self.previousSentence)
                self.previousSentence.append(self.words)
                self.lastSentence += 1
                self.frame.toolbar.EnableTool(ID_FORWARD, 0) 

            self.UpdatePanel(True)

    def OnListBox(self, event):
        self.word = event.GetString()

        if not self.connection:
            self.MakeConnection()
        else:
            self.SearchWord(self.word)

    def OnText(self, event):
        self.currentSentence = event.GetString()

    def OnToolSound(self, event):
        if not event:
            self.frame.toolbar.EnableTool(ID_SOUND, 0)

    def OnToolBackward(self, event):
        if not self.previousSentence:
            self.frame.toolbar.EnableTool(ID_BACKWARD, 0)
            return
        else:
            self.lastSentence -= 1
            self.frame.toolbar.EnableTool(ID_FORWARD, 1)
            self.currentSentence = self.previousSentence[self.lastSentence - 1]
            
            if self.lastSentence == 1:
                self.frame.toolbar.EnableTool(ID_BACKWARD, 0)

        self.UpdatePanel()
         
    def OnToolForward(self, event):
        if not self.previousSentence:
            self.frame.toolbar.EnableTool(ID_FORWARD, 0)
            return
        else:
            self.lastSentence += 1
            self.frame.toolbar.EnableTool(ID_BACKWARD, 1)
            self.currentSentence = self.previousSentence[self.lastSentence - 1]
            
            if self.lastSentence == len(self.previousSentence):
                self.frame.toolbar.EnableTool(ID_FORWARD, 0)
 
        self.UpdatePanel()
        	                   
    def OnToolRefresh(self, event):
        if self.words:
            self.text.Clear()
            self.text.WriteText(string.join(self.words))

        if not self.connection:
            self.MakeConnection()
        else:            
            if self.searchThread:
                if self.connectionThread.IsRunning():
                    self.connectionThread.stop()
            
                self.searchThread = GetDefinition(self,
                                        self.connection,
                                        self.word,
                                        self.database,
                                        self.strategy)
                self.frame.SetStatusText(_("Searching"))
                self.searchThread.start()

    def OnToolStop(self, event):
        if self.searchThread:
            self.searchThread.stop()
            self.frame.SetStatusText(_("Done"))

    def OnUpdateResult(self, event):
        if not event.word:
            self.connection = None
            self.resultWindow.SetPage(_("<h1>Search Failed</h1>"))
            self.frame.SetStatusText(_("Search failed"))
            return
        
        if event.results:
            self.resultWindow.SetPage("")
            self.previousWords[event.word] = event.results
            
            while len(self.previousWords) > self.maxWords:
                self.previousWords.popitem()

            for result in event.results:
                self.resultWindow.AppendToPage(result)
                self.resultWindow.AppendToPage("<hr>")
        else:
            self.resultWindow.SetPage(_("<h1>Definition Not Found</h1>"))

        self.frame.SetStatusText(_("Done"))

    def OnUpdateConnection(self, event):
        self.connection = event.connection

        if self.connection:
            self.frame.SetStatusText(_("Connected"))
            
            if event.onButton:
                self.OnButton(None)
            else:
                self.OnToolRefresh(None)
        else:
            self.frame.SetStatusText(_("Connection failed"))

    def UpdatePanel(self, onButton = False):
        if not onButton:
            for i in range(len(self.words)):
                self.list.Delete(0)

    	    self.words = self.currentSentence
        
        if self.words:
            for word in self.words:
                self.list.Append(word)

            self.text.Clear()
            self.text.WriteText(string.join(self.words))            
            self.word = self.words[0]    
            self.list.SetSelection(0)
 
            if not self.connection:
                self.MakeConnection(onButton)
            else:
                self.SearchWord(self.word)
    	
    def SearchWord(self, word):    
        if self.word in self.previousWords:
            event = UpdateResultEvent(word = self.word, results = self.previousWords[self.word])
            wx.PostEvent(self, event)
        else:
            if self.searchThread and self.searchThread.IsRunning():
                self.searchThread.stop()

            self.searchThread = GetDefinition(self,
                                    self.connection,
                                    self.word,
                                    self.database,
                                    self.strategy)
            self.frame.SetStatusText(_("Searching"))
            self.searchThread.start()

    def MakeConnection(self, onButton = False):
        if self.connectionThread and self.connectionThread.IsRunning():
            self.connectionThread.stop()
        
        self.connectionThread = MakeConnection(self, self.hostname, self.port, onButton)
        self.frame.SetStatusText(_("Connecting to server"))
        self.connectionThread.start()
    	
#----------------------------------------------------------------------------

class MyNotebook(wx.Notebook):
    def __init__(self, parent, id):
        wx.Notebook.__init__(self, parent, id, style = wx.NB_TOP)
        self.frame = parent
        self.defaultServer = self.frame.config.defaultServer
        
        self.Bind(wx.EVT_NOTEBOOK_PAGE_CHANGED, self.OnPageChanged)  
        
    def CreatePanel(self, renew = False):
        self.panelList = []
        self.servers = self.frame.config.servers

        if renew:
            self.DeleteAllPages()

        if self.servers:
            for key in range(len(self.servers)):
                panel = MyPanel(self, -1, *self.servers[key][1:])                
                self.panelList.append(panel)
                self.AddPage(panel, self.servers[key][0])
        else:                
            panel = MyPanel(self, -1, *self.defaultServer[1:])
            self.panelList.append(panel)
            self.AddPage(panel, self.defaultServer[0])
        
        self.SetSelection(0)
        selectedPanel = self.panelList[0]
        selectedPanel.button.SetDefault()

        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolSound, id = ID_SOUND)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolBackward, id = ID_BACKWARD)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolForward, id = ID_FORWARD)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolRefresh, id = ID_REFRESH)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolStop, id = ID_STOP)

    def OnPageChanged(self, event):
        panelNumber = event.GetSelection()
        selectedPanel = self.panelList[panelNumber]
        selectedPanel.button.SetDefault()

        if selectedPanel.lastSentence == len(selectedPanel.previousSentence):
            selectedPanel.frame.toolbar.EnableTool(ID_FORWARD, 0)

            if selectedPanel.lastSentence > 1:
                selectedPanel.frame.toolbar.EnableTool(ID_BACKWARD, 1)
            else:
                selectedPanel.frame.toolbar.EnableTool(ID_BACKWARD, 0)
        elif selectedPanel.lastSentence == 1:
            selectedPanel.frame.toolbar.EnableTool(ID_BACKWARD, 0)
            
            if len(selectedPanel.previousSentence) > 1:
                selectedPanel.frame.toolbar.EnableTool(ID_FORWARD, 1)
            else:
                selectedPanel.frame.toolbar.EnableTool(ID_FORWARD, 0)            
        else:
            selectedPanel.frame.toolbar.EnableTool(ID_BACKWARD, 1)
            selectedPanel.frame.toolbar.EnableTool(ID_FORWARD, 1)

        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolSound, id = ID_SOUND)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolBackward, id = ID_BACKWARD)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolForward, id = ID_FORWARD)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolRefresh, id = ID_REFRESH)
        selectedPanel.frame.toolbar.Bind(wx.EVT_TOOL, selectedPanel.OnToolStop, id = ID_STOP)
        
        event.Skip()

#----------------------------------------------------------------------------

class MyFrame(wx.Frame):
    def __init__(
            self, parent, id, title, pos = wx.DefaultPosition,
            size = (400, 500), style = wx.DEFAULT_FRAME_STYLE):
    	wx.Frame.__init__(self, parent, id, title, pos, size, style)
       	self.stayOnTop = False
        self.GetImages()
        self.SetIcon(self.icon)
        self.tbicon = wx.TaskBarIcon()
        self.tbicon.SetIcon(self.tbIcon, __package__) 

        self.CreateStatusBar()
        self.toolbar = self.CreateToolBar(wx.TB_HORIZONTAL | wx.NO_BORDER | wx.TB_FLAT | wx.TB_TEXT)
        self.toolbar.SetToolBitmapSize((24, 24))

        self.toolbar.AddSimpleTool(ID_SOUND, self.sound, _("Sound"), _("Play sound"))
        self.toolbar.AddSeparator()
        self.toolbar.AddSimpleTool(ID_BACKWARD, self.backward, _("Backward"), _("Go backward"))
        self.toolbar.AddSimpleTool(ID_FORWARD, self.forward, _("Forward"), _("Go forward"))
        self.toolbar.AddSimpleTool(ID_REFRESH, self.refresh, _("Refresh"), _("Refresh window"))
        self.toolbar.AddSimpleTool(ID_STOP, self.stop, _("Stop"), _("Stop resolving"))
        self.toolbar.AddSeparator()
        self.toolbar.AddSimpleTool(ID_PREFERENCES, self.preferences, _("Preferences"), _("Set preferences"))
        self.toolbar.AddSimpleTool(ID_ABOUT, self.about, _("About"), _("About wxDick"))
        self.toolbar.AddSimpleTool(ID_EXIT, self.exit, _("Exit"), _("Exit wxDick"))
        self.toolbar.Realize()

        if wx.Platform == "__WXMSW__":
            self.config = Config(os.path.join("conf", "wxdick.conf"))
        else:
            self.config = Config(os.path.expanduser(os.path.join("~", ".wxdick.conf")))
	
        if self.config.stayOnTop:
            self.SetStayOnTop(None)
        
        self.notebook = MyNotebook(self, -1) 
        self.ApplyPreferences()

        self.Bind(wx.EVT_CLOSE, self.OnTaskHide)
        self.Bind(wx.EVT_TOOL, self.OnToolPreferences, id = ID_PREFERENCES)
        self.Bind(wx.EVT_TOOL, self.OnToolAbout, id = ID_ABOUT)
        self.Bind(wx.EVT_TOOL, self.OnExit, id = ID_EXIT)
        self.tbicon.Bind(wx.EVT_MENU, self.SetStayOnTop, id = ID_STAYONTOP)
        self.tbicon.Bind(wx.EVT_MENU, self.OnTaskShow, id = ID_SHOW)
        self.tbicon.Bind(wx.EVT_MENU, self.OnTaskHide, id = ID_HIDE)
        self.tbicon.Bind(wx.EVT_MENU, self.OnExit, id = ID_EXIT)
        self.tbicon.Bind(wx.EVT_TASKBAR_RIGHT_UP, self.OnTaskMenu)
        self.tbicon.Bind(wx.EVT_TASKBAR_LEFT_DCLICK, self.OnCheckShowHideToggle)
	wx.GetApp().Bind(wx.EVT_UPDATE_UI, self.OnCheckStayOnTop, id = ID_STAYONTOP)

    def GetImages(self, theme = "default"):
        if wx.Platform != "__WXMAC__":
            if wx.Platform == "__WXGTK__":
                dim = 22
            else:
                dim = 16

            if wx.Platform == "__WXMSW__":
                self.tbIcon = wx.IconFromBitmap(
                    wx.Image(os.path.join("pixmaps", theme + "_wxdick.ico"), wx.BITMAP_TYPE_ICO).Scale(dim, dim).ConvertToBitmap())
            else:
                self.tbIcon = wx.IconFromBitmap(
                    wx.Image(os.path.join(sys.prefix, "share", "wxdick", "pixmaps", theme + "_wxdick.png"), wx.BITMAP_TYPE_PNG).Scale(dim, dim).ConvertToBitmap())
            
        if wx.Platform == "__WXMSW__":
            self.icon = wx.Icon(os.path.join("pixmaps", theme + "_wxdick.ico"), wx.BITMAP_TYPE_ICO) 
            self.sound = wx.Image(os.path.join("pixmaps", theme + "_sound.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.backward = wx.Image(os.path.join("pixmaps", theme + "_backward.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.forward = wx.Image(os.path.join("pixmaps", theme + "_forward.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()  
            self.refresh = wx.Image(os.path.join("pixmaps", theme + "_refresh.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.stop = wx.Image(os.path.join("pixmaps", theme + "_stop.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.preferences = wx.Image(os.path.join("pixmaps", theme + "_preferences.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.about = wx.Image(os.path.join("pixmaps", theme + "_about.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.exit = wx.Image(os.path.join("pixmaps", theme + "_exit.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.taskExit = wx.Image(os.path.join("pixmaps", theme + "_task_exit.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
        else:
            prefix = os.path.join(sys.prefix, "share", "wxdick")
            self.icon = wx.Icon(os.path.join(sys.prefix, "share", "wxdick", "pixmaps", theme + "_wxdick.png"), wx.BITMAP_TYPE_PNG) 	
            self.sound = wx.Image(os.path.join(prefix, "pixmaps", theme + "_sound.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.backward = wx.Image(os.path.join(prefix, "pixmaps", theme + "_backward.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.forward = wx.Image(os.path.join(prefix, "pixmaps", theme + "_forward.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()  
            self.refresh = wx.Image(os.path.join(prefix, "pixmaps", theme + "_refresh.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.stop = wx.Image(os.path.join(prefix, "pixmaps", theme + "_stop.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.preferences = wx.Image(os.path.join(prefix, "pixmaps", theme + "_preferences.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.about = wx.Image(os.path.join(prefix, "pixmaps", theme + "_about.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
            self.exit = wx.Image(os.path.join(prefix, "pixmaps", theme + "_exit.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()

    def ApplyPreferences(self, renew = False):
        self.notebook.CreatePanel(renew)
    	
    def OnToolPreferences(self, event):
        dialog = MyPreferences(self, -1, _("Preferences"))
        dialog.CenterOnScreen()
        
        if wx.ID_OK == dialog.ShowModal():
            self.ApplyPreferences(True)

        dialog.Destroy()

    def OnToolAbout(self, event):
        dialog = MyAbout(self, -1, _("About wxDick"))
        dialog.CenterOnScreen()
        dialog.ShowModal()
        dialog.Destroy()

    def OnExit(self,event):
        for panel in self.notebook.panelList:
            if panel.searchThread:
                panel.searchThread.stop()

            if panel.connectionThread:
                panel.connectionThread.stop()

        self.Destroy()
 
    def OnTaskShow(self,event):
        self.Show(True)
         
    def OnTaskHide(self,event):
        self.Show(False)

    def OnCheckShowHideToggle(self,event):
        if self.IsShown():
            self.Show(False)
        else:
            self.Show(True)

    def OnCheckStayOnTop(self, event):
        if self.stayOnTop:
            event.Check(True)
        else:
            event.Check(False)
	
    def SetStayOnTop(self, event):
        if self.stayOnTop:
            self.SetWindowStyle(~wx.STAY_ON_TOP & self.GetWindowStyle())
            self.stayOnTop = False
        else:
            self.SetWindowStyle(wx.STAY_ON_TOP | self.GetWindowStyle())
            self.stayOnTop = True
	    
    def OnTaskMenu(self, event):
        menu = wx.Menu()

	menu.Append(ID_STAYONTOP, _("&Stay on top"), "", wx.ITEM_CHECK)

	if self.IsShown():
            menu.Append(ID_HIDE, _("&Hide wxDick"))
        else:
            menu.Append(ID_SHOW, _("S&how wxDick"))

        menu.AppendSeparator()
        item = wx.MenuItem(menu, ID_EXIT, _("E&xit wxDick"))

        if wx.Platform == "__WXMSW__":
            item.SetBitmap(self.taskExit)
        else:
            item.SetBitmap(self.exit)

        menu.AppendItem(item)
 
        self.tbicon.PopupMenu(menu)
        menu.Destroy()
        
#----------------------------------------------------------------------------

class MyAbout(wx.Dialog):
    def __init__(
            self, parent, ID, title, size = wx.DefaultSize, pos = wx.DefaultPosition, 
            style = wx.DEFAULT_DIALOG_STYLE
            ):
        pre = wx.PreDialog()
        pre.Create(parent, ID, title, pos, size, style)

        self.PostCreate(pre)

        if wx.Platform == "__WXMSW__":
            bitmap = wx.Image(os.path.join("pixmaps", "about.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
        else:
            bitmap = wx.Image(os.path.join(sys.prefix, "share", "wxdick", "pixmaps", "about.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()

        logo = wx.StaticBitmap(self, -1, bitmap, size = (bitmap.GetWidth(), bitmap.GetHeight()))
        button = wx.Button(self, wx.ID_OK, _("OK"))
        button.SetDefault()

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(logo, 0, wx.ALIGN_CENTRE | wx.ALL, 0)
        sizer.Add(button, 0, wx.ALIGN_RIGHT | wx.ALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

#----------------------------------------------------------------------------

class MySplashScreen(wx.SplashScreen):
    def __init__(self):
        if wx.Platform == "__WXMSW__":
            bitmap = wx.Image(os.path.join("pixmaps", "splash.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
        else:
            bitmap = wx.Image(os.path.join(sys.prefix, "share", "wxdick", "pixmaps", "splash.png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()

	wx.SplashScreen.__init__(self, bitmap,
                        wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_TIMEOUT,
                        2000, None, -1)

#----------------------------------------------------------------------------

class MyApp(wx.App):
    def OnInit(self):
        wx.InitAllImageHandlers()
        splash = MySplashScreen()
        splash.Show()

	win = MyFrame(None, -1, __package__)
	win.Show(True)
        self.SetTopWindow(win)

        return True

#----------------------------------------------------------------------------

def main():
    instance = wx.SingleInstanceChecker(__package__)

    if instance.IsAnotherRunning():
        sys.exit(1)
    else:
        app = MyApp(0)
	app.MainLoop()

#----------------------------------------------------------------------------

if __name__ == "__main__":
    main()
